@@ -3,6 +3,8 @@
 #
 # + pack/unpack everywhere
 # + db-sorted iteritems
+# - int and float fields, for sorting
+# - alert about broken ordering of anything but strings
 #
 #
 __all__ = ['sqldict']
@@ -24,9 +26,8 @@ import base64
 from sqlalchemy import create_engine
 
 
-class UDictException(Exception):pass
-class SQLDictException(UDictException): pass
-class SQLDictError(UDictException): pass
+class SQLDictException(Exception): pass
+class SQLDictError(SQLDictException): pass
 
 
 from sqlalchemy import *
@@ -98,7 +99,9 @@ class BaseSQLDict(DictMixin):
         table = Table(self._tablename, self._meta,
             Column('sqldict_ns',  String(nssize), nullable=False, primary_key=True),
             Column('sqldict_key', String(keysize), nullable=False, primary_key=True),
+            Column('sqldict_key_int', Integer, nullable=True),
             Column('sqldict_val', String(valsize), nullable=False),
+            Column('sqldict_val_int', Integer, nullable=True),
             Column('sqldict_flags', Integer, nullable=False),
         )
         self._table = table
@@ -165,6 +168,20 @@ class BaseSQLDict(DictMixin):
             val = str(val)
         return val
 
+    def _get_sort_column(self, flags):
+        table = self._table
+        sort = self._sort
+        col = table.c.sqldict_val
+        if sort == 1:
+            if flags & self._F_VAL_INT:
+                col = table.c.sqldict_val_int
+        if sort == 2:
+            if flags & self._F_KEY_INT:
+                col = table.c.sqldict_key_int
+        return col
+
+
+
     #
     # dict methods
     # 
@@ -181,9 +198,11 @@ class BaseSQLDict(DictMixin):
         query = query.where(table.c.sqldict_ns == ns)
         if sort == 1:
             #print 'sort 1'
+            query = query.order_by(table.c.sqldict_key_int)
             query = query.order_by(table.c.sqldict_key)
         if sort == 2:
             #print 'sort 2'
+            query = query.order_by(table.c.sqldict_val_int)
             query = query.order_by(table.c.sqldict_val)
         #print query
         result = conn.execute(query)
@@ -195,6 +214,51 @@ class BaseSQLDict(DictMixin):
             val = self.unpack_val(val, flags)
             yield (key, val)
 
+    def iterkeys(self):
+        conn = self._conn
+        ns = self._ns
+        table = self._table
+        sort = self._sort
+        #
+        query = select([table.c.sqldict_key,
+                        table.c.sqldict_flags])
+        query = query.where(table.c.sqldict_ns == ns)
+        if sort == 1:
+            query = query.order_by(table.c.sqldict_key_int)
+            query = query.order_by(table.c.sqldict_key)
+        if sort == 2:
+            query = query.order_by(table.c.sqldict_val_int)
+            query = query.order_by(table.c.sqldict_val)
+        result = conn.execute(query)
+        for row in result:
+            key = row['sqldict_key']
+            flags = row['sqldict_flags']
+            key = self.unpack_key(key, flags)
+            yield key
+
+    def itervalues(self):
+        conn = self._conn
+        ns = self._ns
+        table = self._table
+        sort = self._sort
+        #
+        query = select([table.c.sqldict_val,
+                        table.c.sqldict_flags])
+        query = query.where(table.c.sqldict_ns == ns)
+        if sort == 1:
+            query = query.order_by(table.c.sqldict_key_int)
+            query = query.order_by(table.c.sqldict_key)
+        if sort == 2:
+            query = query.order_by(table.c.sqldict_val_int)
+            query = query.order_by(table.c.sqldict_val)
+        result = conn.execute(query)
+        for row in result:
+            val = row['sqldict_val']
+            flags = row['sqldict_flags']
+            val = self.unpack_val(val, flags)
+            yield val
+
+
     def namespaces(self):
         conn = self._conn
         ns = self._ns
@@ -252,9 +316,11 @@ class BaseSQLDict(DictMixin):
         query = query.where(table.c.sqldict_ns == ns)
         if sort == 1:
             #print 'sort 1'
+            query = query.order_by(table.c.sqldict_key_int)
             query = query.order_by(table.c.sqldict_key)
         if sort == 2:
             #print 'sort 2'
+            query = query.order_by(table.c.sqldict_val_int)
             query = query.order_by(table.c.sqldict_val)
         #print query
         result = conn.execute(query)
@@ -338,10 +404,6 @@ def sqldict(dburi='sqlite://', engine=None, conn=None, ns='', tablename='',
         error = 'sqldict(): must specify exactly one of `dburi`, `engine` or `conn`'
         raise SQLDictError(error)
 
-    if not engine and not conn and dburi=='sqlite://':
-        create = 1
-        tablename = 'sqldict_dummy'
-
     if not tablename:
         error = 'sqldict(): must specify `tablename`'
         raise SQLDictError(error)
