@@ -41,6 +41,7 @@ class Test_sqldict(unittest.TestCase):
                 assert val==dict(qwe=456)
                 assert d[key]==val
 
+        print 'ASSERTI', d, len(d)
         assert len(d) == 6
         del d['a']
         assert len(d) == 5
@@ -55,13 +56,17 @@ class Test_sqldict(unittest.TestCase):
         del d[('asd',123)]
         assert len(d) == 0
 
-
+    def test_sqlite_broken(self):
+        #engine = create_engine('mysql://sqldict:balalajkaperestrojka@localhost/sqldict')
+        #conn = engine.connect()
         d = sqldict(dburi='sqlite://', tablename='haha', create=1)
+        self.large(d)
+        d.drop()
 
     def test_mysql1(self):
         engine = create_engine('mysql://sqldict:balalajkaperestrojka@localhost/sqldict')
         conn = engine.connect()
-        d = sqldict(dburi='sqlite://', tablename='haha', create=1)
+        d = sqldict(conn=conn, tablename='haha', create=1)
         self.large(d)
         d.drop()
 
@@ -112,18 +117,75 @@ class Test_sqldict(unittest.TestCase):
             print 'iteritems', key, val
         d.drop()
 
+    def test_iterkeys(self):
+        d = sqldict(tablename='test_iterkeys', create=1)
+        d['a'] = 'asd'
+        d['e'] = 'asd'
+        d['f'] = 'asd'
+        d['b'] = 'asd'
+        d['d'] = 'asd'
+        d['c'] = 'asd'
+        for key in d.iterkeys():
+            print 'KEYKEY', key
+        keys = list(d.iterkeys())
+        assert len(keys) == 6
+        assert keys == sorted(keys)
+        d.drop()
+
+    def test_iterkeys_int(self):
+        d = sqldict(tablename='test_iterkeys', create=1)
+        d[6] = 'asd'
+        d[3] = 'asd'
+        d[5] = 'asd'
+        d[1] = 'asd'
+        d[2] = 'asd'
+        d[4] = 'asd'
+        keys = list(d.iterkeys())
+        assert len(keys) == 6
+        assert keys == sorted(keys)
+        d.drop()
+
+    def test_itervalues_int(self):
+        d = sqldict(tablename='test_iterkeys', create=1, sort=2)
+        d['asd'] = 3
+        d['qwe'] = 6
+        d['foo'] = 2
+        d['bar'] = 1
+        d['sdf'] = 4
+        d['wer'] = 5
+        values = list(d.itervalues())
+        assert len(values) == 6
+        assert values == sorted(values)
+        d.drop()
+
+    def test_itervalues(self):
+        engine = create_engine('mysql://sqldict:balalajkaperestrojka@localhost/sqldict')
+        conn = engine.connect()
+        d = sqldict(conn=conn, tablename='test_itervalues', create=1, sort=2)
+        d['d'] = 'esd'
+        d['a'] = 'asd'
+        d['c'] = 'fsd'
+        d['f'] = 'csd'
+        d['e'] = 'bsd'
+        d['b'] = 'dsd'
+        values = list(d.itervalues())
+        assert values == sorted(values)
+        d.drop()
+
     def test_namespaces(self):
         engine = create_engine('mysql://sqldict:balalajkaperestrojka@localhost/sqldict')
         conn = engine.connect()
         prefix='rr'
         nslist = ['na', 'nb', 'ny', 'nn', 'nm']
+        d = None
         for ns in nslist:
-            d = sqldict(conn=conn, tablename='test_namespaces2',
+            d = sqldict(conn=conn, tablename='xibi',
                     create=1, ns=ns)
+            print d.namespaces()
             d['aa'] = 'asdasd'
             d['bb'] = 'qweqwe'
             d['cc'] = 'rtyrty'
-        print d.namespaces()
+        print 'NAMESPACES', d.namespaces()
         assert d.namespaces() == ['na', 'nb', 'nm', 'nn', 'ny']
         d.drop()
 
